==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/tests/src/Functional/Vpc/InternetGatewayTest.php <==
<?php

namespace Drupal\Tests\aws_cloud\Functional\Vpc;

use Drupal\Tests\aws_cloud\Functional\AwsCloudTestBase;

/**
 * Tests AWS Cloud Internet Gateway.
 *
 * @group AWS Cloud
 */
class InternetGatewayTest extends AwsCloudTestBase {

  public const AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions(): array {
    return [
      'view all cloud service providers',
      'list aws cloud internet gateway',
      'add aws cloud internet gateway',
      'view any aws cloud internet gateway',
      'edit any aws cloud internet gateway',
      'delete any aws cloud internet gateway',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars(): array {
    return [
      'internet_gateway_id' => 'igw-' . $this->getRandomId(),
    ];
  }

  /**
   * Tests CRUD for Internet Gateway information.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Exception
   */
  public function testInternetGateway(): void {
    $cloud_context = $this->cloudContext;

    // List Internet Gateway for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    $this->assertNoErrorMessage();

    // Add a new Internet Gateway.
    $add = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    $this->reloadMockData();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $internet_gateway_id = 'igw-' . $this->getRandomId();
      $this->addInternetGatewayMockData($add[$i], $internet_gateway_id);
      $this->updateCreateInternetGatewayMockData($internet_gateway_id);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/add");
      $this->submitForm(
        $add[$i],
        $this->t('Save'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '%label' => $add[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been created.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($add[$i]['name']);

      $add[$i]['internet_gateway_id'] = $internet_gateway_id;
    }

    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {
      // Make sure the all internet gateway listing exists.
      $this->drupalGet('/clouds/aws_cloud/internet_gateway');
      $this->assertNoErrorMessage();

      for ($j = 0; $j < $num; $j++) {
        $this->assertSession()->pageTextContains($add[$j]['name']);
      }
    }

    // Edit a Internet Gateway.
    $edit = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

      $this->modifyInternetGatewayMockData($i, $edit[$i]);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/edit");
      $this->submitForm(
        $edit[$i],
        $this->t('Save'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '%label' => $edit[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been updated.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Update tags for empty.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      // Update tags.
      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', '', FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Delete name tags.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      // Update tags.
      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', '', TRUE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Update tags.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', $edit[$i]['name'], FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Delete Internet Gateway.
    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/delete");
      $this->assertNoErrorMessage();

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/delete");
      $this->submitForm(
        [],
        $this->t('Delete'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '@label' => $edit[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type @label has been deleted.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests CRUD for Internet Gateway information.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Exception
   */
  public function testUpdateAllInternetGateway(): void {
    $cloud_configs = [];

    // List Internet Gateway for Amazon EC2.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();

    // Create Cloud Config.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->cloudContext = $this->random->name(8);
      $cloud_config = $this->createCloudConfigTestEntity($this->cloudContext);
      $cloud_configs[] = $cloud_config;
    }

    foreach ($cloud_configs ?: [] as $cloud_config) {
      $cloud_context = $cloud_config->getCloudContext();
      // Add a new Internet Gateway.
      $add = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
      for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
        $this->reloadMockData();

        $internet_gateway_id = $this->latestTemplateVars['internet_gateway_id'];
        $this->addInternetGatewayMockData($add[$i], $internet_gateway_id);

        $this->drupalGet("/clouds/aws_cloud/${cloud_context}/internet_gateway/add");
        $this->submitForm(
          $add[$i],
          $this->t('Save'));
        $this->assertNoErrorMessage();
        $t_args = ['@type' => 'Internet Gateway', '%label' => $add[$i]['name']];
        $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been created.', $t_args)));
        $add[$i]['internet_gateway_id'] = $internet_gateway_id;
      }
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Internet Gateway.
    $edit = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $cloud_context = $cloud_config->getCloudContext();
      for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

        $this->modifyInternetGatewayMockData($i, $edit[$i]);

        $this->drupalGet("/clouds/aws_cloud/${cloud_context}/internet_gateway/${num}/edit");
        $this->submitForm(
          $edit[$i],
          $this->t('Save'));
        $this->assertNoErrorMessage();
        $t_args = [
          '@type' => 'Internet Gateway',
          '%label' => $edit[$i]['name'],
        ];
        $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been updated.', $t_args)));
      }
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Delete Internet Gateway in mock data.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->deleteInternetGatewayMockData($i);
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests deleting Internet Gateways with bulk operation.
   *
   * @throws \Exception
   */
  public function testInternetGatewayBulk(): void {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      // Create Internet Gateways.
      $internet_gateways = $this->createInternetGatewaysRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($internet_gateways ?: [] as $internet_gateway) {
        $entities[] = $this->createInternetGatewayTestEntity($index++, $internet_gateway['InternetGatewayId'], $internet_gateway['Name'], $cloud_context);
      }
      $this->runTestEntityBulk('internet_gateway', $entities);
    }
  }

}

==> cloud-4.0.0-rc1.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/tests/src/Functional/Vpc/InternetGatewayTest.php <==
<?php

namespace Drupal\Tests\aws_cloud\Functional\Vpc;

use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\Tests\aws_cloud\Functional\AwsCloudTestBase;

/**
 * Tests AWS Cloud internet gateway.
 *
 * @group AWS Cloud
 */
class InternetGatewayTest extends AwsCloudTestBase {

  public const AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT = 2;
  public const AWS_CLOUD_INTERNET_GATEWAY_ADD_BUTTON = 'Add AWS Cloud internet gateway';

  /**
   * {@inheritdoc}
   */
  protected function getPermissions(): array {
    return [
      'view all cloud service providers',
      'list aws cloud internet gateway',
      'add aws cloud internet gateway',
      'view any aws cloud internet gateway',
      'edit any aws cloud internet gateway',
      'delete any aws cloud internet gateway',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars(): array {
    return [
      'internet_gateway_id' => 'igw-' . $this->getRandomId(),
      'error_code' => Ec2ServiceInterface::DRY_RUN_OPERATION,
    ];
  }

  /**
   * Tests CRUD for Internet Gateway information.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Exception
   */
  public function testInternetGateway(): void {
    $cloud_context = $this->cloudContext;

    // List Internet Gateway for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    $this->assertNoErrorMessage();
    $this->assertSession()->pageTextContains(self::AWS_CLOUD_INTERNET_GATEWAY_ADD_BUTTON);

    // Add a new Internet Gateway.
    $add = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    $this->reloadMockData();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $internet_gateway_id = 'igw-' . $this->getRandomId();
      $this->addInternetGatewayMockData($add[$i], $internet_gateway_id, $cloud_context, $this->webUser->id());
      $this->updateCreateInternetGatewayMockData($internet_gateway_id);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/add");
      $this->submitForm(
        $add[$i],
        $this->t('Save'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '%label' => $add[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been created.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($add[$i]['name']);

      $add[$i]['internet_gateway_id'] = $internet_gateway_id;
    }

    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {
      // Make sure the all internet gateway listing exists.
      $this->drupalGet('/clouds/aws_cloud/internet_gateway');
      $this->assertNoErrorMessage();

      for ($j = 0; $j < $num; $j++) {
        $this->assertSession()->pageTextContains($add[$j]['name']);
      }

      // Make sure uid.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num");
      $this->assertSession()->pageTextContains(
        $this->cloudService->getTagCreatedByUid(
          'aws_cloud',
          $cloud_context
        )
      );
      $this->assertSession()->pageTextContains(
        $this->webUser->getAccountName()
      );
    }

    // Edit a Internet Gateway.
    $edit = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

      $this->modifyInternetGatewayMockData($i, $edit[$i], $cloud_context, $this->webUser->id());

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/edit");
      $this->submitForm(
        $edit[$i],
        $this->t('Save'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '%label' => $edit[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been updated.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextContains($edit[$i]['name']);

      // Make sure uid.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num");
      $this->assertSession()->pageTextContains(
        $this->cloudService->getTagCreatedByUid(
          'aws_cloud',
          $cloud_context
        )
      );
      $this->assertSession()->pageTextContains(
        $this->webUser->getAccountName()
      );
    }

    // Update tags for empty.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      // Update tags.
      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', '', FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Delete name tags.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      // Update tags.
      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', '', TRUE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Update tags.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {

      $this->updateTagsInMockData($i, 'InternetGateways', 'Name', $edit[$i]['name'], FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['internet_gateway_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Delete Internet Gateway.
    for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/delete");
      $this->assertNoErrorMessage();

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/$num/delete");
      $this->submitForm(
        [],
        $this->t('Delete'));
      $this->assertNoErrorMessage();
      $t_args = ['@type' => 'Internet Gateway', '@label' => $edit[$i]['name']];
      $this->assertSession()->pageTextContains(strip_tags($this->t('The @type @label has been deleted.', $t_args)));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests CRUD for Internet Gateway information.
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Exception
   */
  public function testUpdateAllInternetGateway(): void {
    $cloud_configs = [];

    // List Internet Gateway for Amazon EC2.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();

    // Create Cloud Config.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->cloudContext = $this->random->name(8);
      $cloud_config = $this->createCloudConfigTestEntity($this->cloudContext);
      $cloud_configs[] = $cloud_config;
    }

    foreach ($cloud_configs ?: [] as $cloud_config) {
      $cloud_context = $cloud_config->getCloudContext();
      // Add a new Internet Gateway.
      $add = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
      for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {
        $this->reloadMockData();

        $internet_gateway_id = $this->latestTemplateVars['internet_gateway_id'];
        $this->addInternetGatewayMockData($add[$i], $internet_gateway_id, $cloud_context, $this->webUser->id());

        $this->drupalGet("/clouds/aws_cloud/${cloud_context}/internet_gateway/add");
        $this->submitForm(
          $add[$i],
          $this->t('Save'));
        $this->assertNoErrorMessage();
        $t_args = ['@type' => 'Internet Gateway', '%label' => $add[$i]['name']];
        $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been created.', $t_args)));
        $add[$i]['internet_gateway_id'] = $internet_gateway_id;
      }
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Internet Gateway.
    $edit = $this->createInternetGatewayTestFormData(self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT);
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $cloud_context = $cloud_config->getCloudContext();
      for ($i = 0, $num = 1; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++, $num++) {

        $this->modifyInternetGatewayMockData($i, $edit[$i], $cloud_context, $this->webUser->id());

        $this->drupalGet("/clouds/aws_cloud/${cloud_context}/internet_gateway/${num}/edit");
        $this->submitForm(
          $edit[$i],
          $this->t('Save'));
        $this->assertNoErrorMessage();
        $t_args = [
          '@type' => 'Internet Gateway',
          '%label' => $edit[$i]['name'],
        ];
        $this->assertSession()->pageTextContains(strip_tags($this->t('The @type %label has been updated.', $t_args)));
      }
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Delete Internet Gateway in mock data.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->deleteInternetGatewayMockData($i);
    }

    // Make sure listing.
    $this->drupalGet('/clouds/aws_cloud/internet_gateway');
    $this->assertNoErrorMessage();
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    foreach ($cloud_configs ?: [] as $cloud_config) {
      $this->assertSession()->pageTextContains($this->t('Updated @resources of @cloud_config cloud service provider.', [
        '@resources' => 'Internet Gateways',
        '@cloud_config' => $cloud_config->getName(),
      ]));
    }

    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests deleting Internet Gateways with bulk operation.
   *
   * @throws \Exception
   */
  public function testInternetGatewayBulk(): void {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      // Create Internet Gateways.
      $internet_gateways = $this->createInternetGatewaysRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($internet_gateways ?: [] as $internet_gateway) {
        $entities[] = $this->createInternetGatewayTestEntity($index++, $internet_gateway['InternetGatewayId'], $internet_gateway['Name'], $cloud_context);
      }
      $this->runTestEntityBulk('internet_gateway', $entities);
    }
  }

  /**
   * Tests hiding buttons based on access control.
   */
  public function testInternetGatewayAccessControl(): void {
    $cloud_context = $this->cloudContext;

    for ($i = 0; $i < self::AWS_CLOUD_INTERNET_GATEWAY_REPEAT_COUNT; $i++) {
      $this->updateErrorCodeMockData(['CreateInternetGateway']);
      // List Internet Gateway for Amazon EC2.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway");
      $this->assertNoErrorMessage();
      $this->assertSession()->pageTextNotContains(self::AWS_CLOUD_INTERNET_GATEWAY_ADD_BUTTON);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/internet_gateway/add");
      $this->assertAccessDenied();
    }
  }

}
